/*      YEAR_CAL.C

<t>     Produce an Annual Calendar
        From Computer Language, Dec/90
*/
#include <stdio.h>
#include <stdlib.h>
#include "year_cal.h"


DATE    julian_date( int day, int month, int year )
/*
        Calculate the Julian day number for the specified day, month, and year
        If the year is B.C., it should be passed in as negative
*/
{
    int   a,b;
    float year_corr;

    /* Correct for negative year */
    year_corr = (year > 0) ? (float) 0.0 : (float) 0.75 ;
    if ( month <= 2 )
    {
        year  -=  1 ;
        month += 12 ;
    }
    b = 0;
    /* Cope with the Gregorian calendar reforms */
    if ( year * 10000.0 + month * 100.0 + day >= 15821015.0 )
    {
        a = year / 100;
        b = 2 - a + a / 4;
    }
    return( (DATE) (long) (365.25 * year - year_corr) + (long) (30.6001 * (month + 1)) + day + 1720994L + b );
}


DATE    easter( int year )
/*
        Calculate the Julian day number of Easter for the
        specified year, which cannot be negative.
*/
{
    int a, b, c, d, e, f, g, h, i, k, l, m, n, p, day, month;

    if (year >= 1583)
    {
        /* Calculate date of Easter for a year in the Gregorian calendar. */
        a = year % 19;
        b = year / 100;
        c = year % 100;
        d = b / 4;
        e = b % 4;
        f = (b + 8) / 25;
        g = (b - f + 1) / 3;
        h = (19 * a + b - d - g + 15) % 30;
        i = c / 4;
        k = c % 4;
        l = (32 + 2 * e + 2 * i - h - k) % 7;
        m = (a + 11 * h + 22 * l) / 451;
        n = (h + l - 7 * m + 114) / 31;
        p = (h + l - 7 * m + 114) % 31;

        month = n;
        day = p + 1;
    }
    else
    {
        /* Calculate date of Easter for a year in the Julian calendar. */
        a = year % 4;
        b = year % 7;
        c = year % 19;
        d = (19 * c + 15) % 30;
        e = (2 * a + 4 * b - d + 34) % 7;
        f = (d + e + 114) / 31;
        g = (d + e + 114) % 31;

        month = f;
        day = g + 1;
    }
    return( julian_date(day, month, year) );
}


void    calendar_date( DATE jdate, int* day, int* month, int* year )
/*
        Calculate the day, month, and year corresponding to a Julian date number
        The year will be negative if it is B.C.
*/
{
    long a, b, c, d, e, z, alpha;

    z = jdate + 1;
    /* Cope with Gregorian calendar reform */
    if ( z < 2299161L)
    {
        a = z;
    }
    else
    {
        alpha = (long) ((z - 1867216.25) / 36524.25);
        a = z + 1 + alpha - alpha / 4;
    }
    b = a + 1524;
    c = (long) ((b - 122.1) / 365.25);
    d = (long) (365.25 * c);
    e = (long) ((b - d) / 30.6001);
    *day = (int) (b - d - (long) (30.6001 * e)) ;
    *month = (int) ((float) e < (float) 13.5)     ? ((int) (e - 1))    : ((int) (e - 13))   ;
    *year  = (int) ((float) *month > (float) 2.5) ? ((int) (c - 4716)) : ((int) (c - 4715)) ;
}


int     valid_date( int day, int month, int year )
/*
        Validate the specified day, month, and year.

        For example, the following dates are not valid:

        February 29, 1973 (not a leap year ) April 31 (April has only 30 days)
*/
{
    int cal_day, cal_month, cal_year;

    // Convert date to Julian day number and back.
    calendar_date(julian_date(day, month, year), &cal_day, &cal_month, &cal_year);

    // Date is valid if day, month, and year did not change.
    return( (day == cal_day) && (month == cal_month) && (year == cal_year) );
}


int     day_of_week( DATE date )
/*
        Calculate the day of the week for the specified Julian day number:

        1 = Sunday, 2 = Monday, ..., 7 = Saturday
*/
{
    return( (int) ((date + 2) % 7 + 1) );
}


void    month_calendar( int month, int year )
/*
        Print a calendar for the specified month.
*/

{
    DATE b_date, e_date, c_date;
    int day, c_day, c_month, c_year;

    printf( "\n" );

    /* Calculate Julian day numbers for first and last days of the month. */
    b_date = julian_date(1, month, year);

    if (month < 12)
        e_date = julian_date(1, month + 1, year) - 1;
    else
        e_date = julian_date(1, 1, year + 1) - 1;

    printf( "%2d/%d:\n\n", month, year );
    printf( "SUN MON TUE WED THU FRI SAT\n\n" );

    /* Put first day of month under the correct day column. */
    for ( day = 1 ; day < day_of_week(b_date) ; day++ )
        printf( "    " );

    /* Print all the days */
    for ( c_date = b_date ; c_date <= e_date ; c_date++ )
    {
        calendar_date( c_date, &c_day, &c_month, &c_year );
        printf( "%3u ", c_day );

        if ( day_of_week(c_date) == 7 )
            printf( "\n" );
    }

    /* If Easter happens this month, print the date. */
    if ( (year >= 0) && (easter(year) >= b_date && easter(year) <= e_date) )
    {
        calendar_date( easter(year), &c_day, &c_month, &c_year );
        printf( "\nEaster: %2d/%2d", c_month, c_day );
    }
    printf( "\n---------------------------\n" );
}


void    year_calendar( int year )
/* Print a one-year calendar for the specified year. */
{
    int month;

    for ( month = 1 ; month <= 12 ; month++ )
        month_calendar( month, year );
}


void    main( int argc, char *argv[] )
{
    if (argc != 2)
    {
        printf("calendar: usage: calendar <year>\n");
        exit(-1);
    }

    /* Generate a one-year calendar for the specified year. */
    year_calendar(atoi(argv[1]));
}
